@@ -25,6 +25,7 @@ use Data::Dumper;
 use List::Util qw(shuffle);
 use Bio::Align::Utilities qw(:all);
 use Exporter ();
+use Bio::SearchIO;
 
 if ($ENV{'DEBUG'}) { use Data::Dumper }
 
@@ -56,28 +57,28 @@ my $VERSION = $Bio::BPWrapper::VERSION;
 ## a reference to the handler subroutine (defined below), and test
 ## that it works.
 my %opt_dispatch = (
-    "avpid" => \&print_avp_id,
-    "bootstrap" => \&bootstrap,
+    "avg-pid" => \&print_avp_id,
+    "boot" => \&bootstrap,
     "codon-view" => \&draw_codon_view,
     "delete" => \&del_seqs,
-    "nogaps" => \&remove_gaps,
+    "no-gaps" => \&remove_gaps,
     "length" => \&print_length,
     "match" => \&print_match,
-    "numseq" => \&print_num_seq,
+    "num-seq" => \&print_num_seq,
     "pick" => \&pick_seqs,
-    "refseq" => \&change_ref,
+    "ref-seq" => \&change_ref,
     "slice" => \&aln_slice,
     "split-cdhit" => \&split_cdhit,
     "uniq" => \&get_unique,
-    "varsites" => \&variable_sites,
+    "var-sites" => \&variable_sites,
     "window" => \&avg_id_by_win,
     "concat" => \&concat,
-    "conblocks" => \&conserved_blocks,
+    "con-blocks" => \&conserved_blocks,
     "consensus" => \&get_consensus,
     "dna2pep" => \&dna_to_protein,
-    "erasecol" => \&remove_gapped_cols_in_one_seq,
+    "rm-col" => \&remove_gapped_cols_in_one_seq,
     "aln-index" => \&colnum_from_residue_pos,
-    "listids" => \&list_ids,
+    "list-ids" => \&list_ids,
     "permute-states" => \&permute_states,
     "pep2dna" => \&protein_to_dna,
     "resample" => \&sample_seqs,
@@ -85,10 +86,10 @@ my %opt_dispatch = (
     "select-third" => \&select_third_sites,
     "remove-third" => \&remove_third_sites,
     "random-slice" => \&random_slice,
-    "uppercase" => \&upper_case,
-    "gapstates" => \&gap_states,
-    "gapstates2" => \&gap_states_matrix,
-    "trimends" => \&trim_ends,
+    "upper" => \&upper_case,
+    "gap-states" => \&gap_states,
+    "gap-states2" => \&gap_states_matrix,
+    "trim-ends" => \&trim_ends,
     "bin-inform" => \&binary_informative,
     "phy-nonint" => \&phylip_non_interleaved
    );
@@ -108,7 +109,6 @@ Call this right after setting up an options hash.
 
 Sets package variables: C<$in_format>, C<$binary>, C<$out_format>, and C<$out>.
 
-
 =cut
 
 sub initialize {
@@ -129,9 +129,19 @@ sub initialize {
 	       while ($aln=$in->next_aln()) { push @alns, $aln }
 	   }
     } else {
-	   $file = shift @ARGV || "STDIN";    # If no more arguments were given on the command line,
+	$file = shift @ARGV || "STDIN";    # If no more arguments were given on the command line,
+	if ($opts{"input"} && $opts{"input"} =~ /blast/) { # "blastxml" (-outfmt 5 ) preferred
+	    my $searchio = Bio::SearchIO->new( -format => $opts{'input'}, ($file eq "STDIN")? (-fh => \*STDIN) : (-file => $file));
+	    while ( my $result = $searchio->next_result() ) {
+		while( my $hit = $result->next_hit ) {
+ 		    my $hsp = $hit->next_hsp; # get first hit; others ignored
+		    $aln = $hsp->get_aln();
+		}
+	    }
+	} else {
 	   $in = Bio::AlignIO->new(-format => $in_format, ($file eq "STDIN")? (-fh => \*STDIN) : (-file => $file));
 	   $aln = $in->next_aln()
+	}
     }
 
     $binary = $opts{"binary"} ? 1 : 0;
@@ -199,7 +209,7 @@ sub write_out($) {
     }
 
 
-    $aln->set_displayname_flat() unless $opts{"noflatname"};
+    $aln->set_displayname_flat() unless $opts{"no-flat"};
     if ($out_format eq 'paml') { &write_out_paml($aln) }
     else { $out->write_aln($aln) }
 }
@@ -640,13 +650,13 @@ L<Bio::SimpleAlign-E<gt>set_new_reference()|https://metacpan.org/pod/Bio::Simple
 =cut
 
 sub change_ref {
-    $aln = $aln->set_new_reference($opts{"refseq"})
+    $aln = $aln->set_new_reference($opts{"ref-seq"})
 }
 
 
 =head2 aln_slice()
 
-Get a slice of the alignment.  The slice is specifiedn
+Get a slice of the alignment.  The slice is specified
 C<$opts{"slice"}> which is set via L<C<#initilize(\%opts)>|/initialize>.
 
 Wraps
@@ -816,7 +826,7 @@ sub concat {
 sub conserved_blocks {
     my $len=$aln->length();
     my $nseq = $aln->num_sequences();
-    my $min_block_size = $opts{"conblocks"};
+    my $min_block_size = $opts{"con-blocks"};
     my %seq_ids;
 
     die "Alignment contains only one sequence: $file\n" if $nseq < 2;
@@ -899,7 +909,7 @@ sub dna_to_protein {
 }
 
 sub remove_gapped_cols_in_one_seq {
-    my $id = $opts{"erasecol"};
+    my $id = $opts{"rm-col"};
     my $nmatch=0;
     my $ref_seq;
     foreach ($aln->each_seq) {
@@ -1018,8 +1028,8 @@ sub sample_seqs {
 
 =head2 shuffle_sites()
 
-Make a shuffled (not bootstraped) alignment. This operation randomizes
-alignment columns. It is used for testing the signficance of long-runs
+Make a shuffled (not bootstrapped) alignment. This operation randomizes
+alignment columns. It is used for testing the significance of long-runs
 of conserved sites in an alignment (e.g., conserved intergenic spacers
 [IGSs]).
 
@@ -1310,7 +1320,7 @@ Here is how to extend.  We'll use option C<--avpid> as an example.
 =over 4
 
 =item *
-Create a new method like one of the above in the previos section.
+Create a new method like one of the above in the previous section.
 
 =item *
 Document your method in pod using C<=head2>. For example:
@@ -1365,7 +1375,7 @@ L<bioaln>: command-line tool for using this
 
 =item *
 
-L<Qui Lab wiki page|http://diverge.hunter.cuny.edu/labwiki/Bioutils>
+L<Qiu Lab wiki page|http://diverge.hunter.cuny.edu/labwiki/Bioutils>
 
 =item *
 
